/*
 * File: \project\routes\objectives\todo.js                                    *
 */

const Schemas = require("../schemas");

/**
 * A plugin (module) that provides common and encapsulated routes
 * @param {FastifyInstance} fastify encapsulated fastify instance
 * @param {Object} options plugin options
 */

async function routes(fastify, options) {
  const endpoint = "/todos";
  const collection = fastify.mongo.db.collection("todos");

  // GET all todos
  fastify.get(
    endpoint + "/all",
    { schema: Schemas.Todo.GET_ALL },
    async (req, res) => {
      const todos = await collection.find({}).toArray();
      res.send({
        message: "All todos retrieved successfully",
        value: todos,
      });
    }
  );

  // GET todo by ID
  fastify.get(
    endpoint + "/:id",
    { schema: Schemas.Todo.GET_BY_ID },
    async (req, res) => {
      const { id } = req.params;
      const todo = await collection.findOne({
        _id: new fastify.mongo.ObjectId(id),
      });

      if (!todo) {
        res.code(404).send({
          message: "Todo not found",
          value: {},
        });
        return;
      }

      res.send({
        message: "Todo retrieved successfully",
        value: todo,
      });
    }
  );

  // POST new todo
  fastify.post(endpoint, { schema: Schemas.Todo.POST }, async (req, res) => {
    const todo = {
      ...req.body,
      time_created: new Date().toISOString(),
      status_changed_at: new Date().toISOString(),
      done: false,
    };

    try {
      const result = await collection.insertOne(todo);
      res.code(201).send({
        message: "Todo created successfully",
        value: result.insertedId.toString(),
      });
    } catch (error) {
      res.code(400).send({
        message: "Invalid todo data",
        value: {},
      });
    }
  });

  // GET filtered todos
  fastify.get(
    endpoint + "/filter",
    { schema: Schemas.Todo.GET_ALL },
    async (req, res) => {
      const { title, categories, done } = req.query;
      const filter = {};

      if (title) {
        filter.title = { $regex: title, $options: "i" };
      }
      if (categories) {
        filter.categories = {
          $in: Array.isArray(categories) ? categories : [categories],
        };
      }
      if (done !== undefined) {
        filter.done = done === "true";
      }

      const todos = await collection.find(filter).toArray();
      res.send({
        message: "Filtered todos retrieved successfully",
        value: todos,
      });
    }
  );
}

module.exports = routes;
